@@ -3,13 +3,14 @@ use pyo3::basic::CompareOp;
 use pyo3::types::PyType;
 use pyo3::{prelude::*, PyMappingProtocol, PyNumberProtocol, PyObjectProtocol};
 
+use crate::monomorphs::RaggedBufferI64;
 use crate::ragged_buffer::RaggedBuffer;
 
 use super::IndicesOrInt;
 
 #[pyclass]
 #[derive(Clone)]
-pub struct RaggedBufferF32(RaggedBuffer<f32>);
+pub struct RaggedBufferF32(pub RaggedBuffer<f32>);
 
 #[pymethods]
 impl RaggedBufferF32 {
@@ -50,12 +51,30 @@ impl RaggedBufferF32 {
     fn size0(&self) -> usize {
         self.0.size0()
     }
-    fn size1(&self, i: usize) -> PyResult<usize> {
-        self.0.size1(i)
+    fn size1(&self, py: Python, i: Option<usize>) -> PyResult<PyObject> {
+        match i {
+            Some(i) => self.0.size1(i).map(|s| s.into_py(py)),
+            None => Ok(self.0.lengths(py).into_py(py)),
+        }
     }
     fn size2(&self) -> usize {
         self.0.size2()
     }
+    fn indices(&self, dim: usize) -> PyResult<RaggedBufferI64> {
+        Ok(RaggedBufferI64(self.0.indices(dim)?))
+    }
+    fn flat_indices(&self) -> PyResult<RaggedBufferI64> {
+        Ok(RaggedBufferI64(self.0.flat_indices()?))
+    }
+    #[classmethod]
+    fn cat(
+        _cls: &PyType,
+        lhs: &RaggedBufferF32,
+        rhs: &RaggedBufferF32,
+        dim: usize,
+    ) -> PyResult<Self> {
+        Ok(RaggedBufferF32(RaggedBuffer::cat(&lhs.0, &rhs.0, dim)?))
+    }
 }
 
 #[pyproto]
@@ -79,10 +98,19 @@ impl PyObjectProtocol for RaggedBufferF32 {
     }
 }
 
+#[derive(FromPyObject)]
+pub enum RaggedBufferF32OrF32 {
+    RB(RaggedBufferF32),
+    Scalar(f32),
+}
+
 #[pyproto]
 impl PyNumberProtocol for RaggedBufferF32 {
-    fn __add__(lhs: RaggedBufferF32, rhs: RaggedBufferF32) -> PyResult<RaggedBufferF32> {
-        Ok(RaggedBufferF32(lhs.0.add(&rhs.0)?))
+    fn __add__(lhs: RaggedBufferF32, rhs: RaggedBufferF32OrF32) -> PyResult<RaggedBufferF32> {
+        match rhs {
+            RaggedBufferF32OrF32::RB(rhs) => Ok(RaggedBufferF32(lhs.0.add(&rhs.0)?)),
+            RaggedBufferF32OrF32::Scalar(rhs) => Ok(RaggedBufferF32(lhs.0.add_scalar(rhs))),
+        }
     }
 }
 
